@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -43,23 +43,21 @@ namespace Common {
   std::string VersionInfo( bool ); // obsolete
 
   inline int look_ahead( std::istream &is ){
-    int nc;
     while( is ){
-      nc=is.peek();
+      int nc=is.peek();
       if ( !isspace(nc) )
 	return nc;
-      nc = is.get();
+      is.get();
     }
     return -1;
   }
 
   inline void skip_spaces( std::istream &is ){
-    int nc;
     while( is ){
-      nc=is.peek();
+      int nc=is.peek();
       if ( !isspace(nc) )
 	return;
-      nc = is.get();
+      is.get();
     }
   }
 
