@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -40,7 +40,7 @@ namespace Timbl {
   bool compare_nocase( const std::string&, const std::string& );
   bool compare_nocase_n( const std::string&, const std::string& );
 
-  std::string StrToCode( const std::string& );
+  std::string StrToCode( const std::string&, bool=true );
   std::string CodeToStr( const std::string& );
 
   std::string correct_path( const std::string&,
