@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -204,7 +204,6 @@ namespace Timbl {
   }
 
   double ValueDistribution::Entropy() const {
-    double Prob = 0.0;
     double entropy = 0.0;
     size_t TotalVals = total_items;
     if ( TotalVals > 0 ){
@@ -213,7 +212,7 @@ namespace Timbl {
       while ( it != distribution.end() ){
 	size_t Freq = it->second->Freq();
 	if ( Freq > 0 ){
-	  Prob = Freq / (double)TotalVals;
+	  double Prob = Freq / (double)TotalVals;
 	  entropy += Prob * Log2(Prob);
 	}
 	++it;
@@ -246,7 +245,8 @@ namespace Timbl {
 	it->second->SetWeight( it->second->Weight() + factor );
       }
       else {
-	distribution[id] = new Vfield( (TargetValue*)val, 1, factor );
+	distribution[id] = new Vfield( reinterpret_cast<TargetValue*>(val),
+				       1, factor );
       }
     }
     total_items += targ->ValuesArray.size();
@@ -255,7 +255,7 @@ namespace Timbl {
 
   void WValueDistribution::Normalize_2( ) {
     for ( const auto& d : distribution ){
-      d.second->SetWeight( log( d.second->Weight() + 1 ));
+      d.second->SetWeight( log1p( d.second->Weight() ) );
     }
     Normalize();
   }
@@ -684,7 +684,7 @@ namespace Timbl {
 
   struct D_D {
     D_D(): dist(0), value(0.0) {};
-    D_D( FeatureValue *fv ){
+    explicit D_D( FeatureValue *fv ): value(0.0) {
       if ( !TiCC::stringTo<double>( fv->Name(), value ) )
 	throw( logic_error("called DD with an non-numeric value" ) );
       dist = &fv->TargetDist;
@@ -817,9 +817,9 @@ namespace Timbl {
     while ( it != ValuesArray.end() ){
       FeatureValue *fv = (FeatureValue*)*it;
       // Entropy for this FV pair.
-      double FVEntropy = 0.0;
       size_t Freq = fv->ValFreq();
       if ( Freq > 0 ){
+	double FVEntropy = 0.0;
 	ValueDistribution::dist_iterator It = fv->TargetDist.begin();
 	while ( It != fv->TargetDist.end() ){
 	  Prob = It->second->Freq() / (double)Freq;
@@ -863,7 +863,6 @@ namespace Timbl {
 				     Target *Targets ){
     chi_square = 0.0;
     long int n_dot_dot = 0;
-    double tmp;
     size_t Size = Targets->ValuesArray.size();
     if ( !n_dot_j ) {
       n_dot_j = new long int[Size];
@@ -905,12 +904,12 @@ namespace Timbl {
 	size_t n = 0;
 	while ( It != fv->TargetDist.end() && n < Size ){
 	  while ( n < It->second->Index()-1 ){
-	    tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
+	    double tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
 	      (double)n_dot_dot;
 	    chi_square += tmp;
 	  }
 	  if ( n == It->second->Index()-1 ){
-	    tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
+	    double tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
 	      (double)n_dot_dot;
 	    if ( fabs(tmp) > Epsilon){
 	      chi_square += ( (tmp - It->second->Freq()) *
@@ -922,7 +921,7 @@ namespace Timbl {
 	    break;
 	}
 	while ( n < Size ){
-	  tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
+	  double tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
 	    (double)n_dot_dot;
 	  chi_square += tmp;
 	}
@@ -933,7 +932,6 @@ namespace Timbl {
   void Feature::ChiSquareStatistics( Target *Targets ){
     chi_square = 0.0;
     long int n_dot_dot = 0;
-    double tmp;
     size_t Size = Targets->ValuesArray.size();
     size_t Num_Vals = ValuesArray.size();
     if ( !n_dot_j ) {
@@ -985,12 +983,12 @@ namespace Timbl {
 	  size_t id = It->second->Index()-1;
 	  long int fr = It->second->Freq();
 	  while ( n < id ){
-	    tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
+	    double tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
 	      (double)n_dot_dot;
 	    chi_square += tmp;
 	  }
 	  if ( n == id ){
-	    tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
+	    double tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
 	      (double)n_dot_dot;
 	    if ( fabs(tmp) > Epsilon ){
 	      chi_square += ( (tmp - fr ) * (tmp - fr ) ) / tmp;
@@ -1001,7 +999,7 @@ namespace Timbl {
 	    break;
 	}
 	while ( n < Size ){
-	  tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
+	  double tmp = ((double)n_dot_j[n++] * (double)n_i_dot[m]) /
 	    (double)n_dot_dot;
 	  chi_square += tmp;
 	}
@@ -1013,9 +1011,9 @@ namespace Timbl {
 
   double Feature::fvDistance( FeatureValue *F, FeatureValue *G,
 			      size_t limit ) const {
-    bool dummy;
     double result = 0.0;
     if ( F != G ){
+      bool dummy;
       if ( metric->isStorable() && matrixPresent( dummy )&&
 	   F->ValFreq() >= matrix_clip_freq &&
 	   G->ValFreq() >= matrix_clip_freq ){
@@ -1054,10 +1052,7 @@ namespace Timbl {
     // otherwise 1. Special case when reading the TopDistribution.
     //
     ValueDistribution *result = 0;
-    TargetValue *target;
     string buf;
-    size_t freq;
-    double sw;
     char nextCh;
     is >> nextCh;   // skip {
     if ( nextCh != '{' ){
@@ -1066,13 +1061,16 @@ namespace Timbl {
     else {
       int next;
       do {
+	size_t freq;
 	is >> ws >> buf;
 	is >> freq;
+	TargetValue *target;
 	if ( do_fr ){
 	  target = Targ->add_value( buf, freq );
 	}
-	else
+	else {
 	  target = Targ->Lookup( buf );
+	}
 	if ( !target ){
 	  delete result;
 	  result = 0;
@@ -1080,8 +1078,9 @@ namespace Timbl {
 	}
 	next = look_ahead(is);
 	if ( next == ',' ){
-	  if ( !result )
+	  if ( !result ) {
 	    result = new ValueDistribution();
+	  }
 	  result->SetFreq( target, freq );
 	  is >> nextCh;
 	  next = look_ahead(is);
@@ -1094,6 +1093,7 @@ namespace Timbl {
 	else if ( isdigit(next) ){
 	  if ( !result )
 	    result = new WValueDistribution();
+	  double sw;
 	  is >> sw;
 	  result->SetFreq( target, freq, sw );
 	  next = look_ahead(is);
@@ -1123,10 +1123,6 @@ namespace Timbl {
     // if do_f we also adjust the value of Frequency of the Target, which is
     // otherwise 1. Special case when reading the TopDistribution.
     //
-    TargetValue *target;
-    size_t freq;
-    unsigned int index;
-    double sw;
     char nextCh;
     is >> nextCh;   // skip {
     if ( nextCh != '{' ){
@@ -1135,8 +1131,11 @@ namespace Timbl {
     else {
       int next;
       do {
+	unsigned int index;
+	size_t freq;
 	is >> index;
 	is >> freq;
+	TargetValue *target;
 	if ( do_fr ){
 	  target = Targ->add_value( index, freq );
 	}
@@ -1161,6 +1160,7 @@ namespace Timbl {
 	  result->SetFreq( target, freq );
 	}
 	else if ( isdigit(next) ){
+	  double sw;
 	  is >> sw;
 	  if ( !result )
 	    result = new WValueDistribution();
@@ -1238,7 +1238,7 @@ namespace Timbl {
     if ( index ) {
       IVCmaptype::const_iterator it = ValuesMap.find( index );
       if ( it != ValuesMap.end() )
-	result = (FeatureValue *)(it->second);
+	result =  reinterpret_cast<FeatureValue*>(it->second);
     }
     return result;
   }
@@ -1266,7 +1266,7 @@ namespace Timbl {
     else {
       it->second->IncValFreq( freq );
     }
-    FeatureValue *result = (FeatureValue *)ValuesMap[index];
+    FeatureValue *result =  reinterpret_cast<FeatureValue*>(ValuesMap[index]);
     if ( tv )
       result->TargetDist.IncFreq(tv, freq );
     return result;
@@ -1333,9 +1333,10 @@ namespace Timbl {
 
   BaseFeatTargClass::BaseFeatTargClass( const BaseFeatTargClass& in ):
     MsgClass( in ),
-    TokenTree( in.TokenTree ){
-    ValuesArray = in.ValuesArray;
-    ValuesMap = in.ValuesMap;
+    ValuesArray( in.ValuesArray ),
+    ValuesMap( in.ValuesMap ),
+    TokenTree( in.TokenTree )
+  {
     is_copy = true;
   }
 
@@ -1355,14 +1356,14 @@ namespace Timbl {
     size_t index = TokenTree->Lookup( str );
     if ( index ) {
       IVCmaptype::const_iterator it = ValuesMap.find( index );
-      result = (TargetValue *)it->second;
+      result = reinterpret_cast<TargetValue*>(it->second);
     }
     return result;
   }
 
   TargetValue *Target::ReverseLookup( size_t index ) const {
     IVCmaptype::const_iterator it = ValuesMap.find( index );
-    return dynamic_cast<TargetValue *>(it->second);
+    return reinterpret_cast<TargetValue *>(it->second);
   }
 
   Feature::~Feature(){
@@ -1398,12 +1399,11 @@ namespace Timbl {
 
   FeatVal_Stat Feature::prepare_numeric_stats(){
     double tmp;
-    size_t freq;
     bool first = true;
     VCarrtype::const_iterator it = ValuesArray.begin();
     while ( it != ValuesArray.end() ){
       FeatureValue *fv = (FeatureValue*)*it;
-      freq = fv->ValFreq();
+      size_t freq = fv->ValFreq();
       if ( freq > 0 ){
 	if ( !TiCC::stringTo<double>( fv->Name(), tmp ) ){
 	  Warning( "a Non Numeric value '" +
@@ -1446,7 +1446,7 @@ namespace Timbl {
     double sum = 0.0;
     vector<double> store( ValuesArray.size() );
     for ( unsigned int i=0; i < ValuesArray.size(); ++i ){
-      FeatureValue *FV = (FeatureValue *)ValuesArray[i];
+      FeatureValue *FV =  reinterpret_cast<FeatureValue*>(ValuesArray[i]);
       double val = TiCC::stringTo<double>( FV->Name() );
       store[i] = val;
       sum += val;
@@ -1476,7 +1476,7 @@ namespace Timbl {
   }
 
   bool Feature::setMetricType( const MetricType M ){
-    if ( !metric || ( metric && M != metric->type() ) ){
+    if ( !metric || M != metric->type() ){
       delete metric;
       metric = getMetricClass(M);
       return true;
@@ -1504,9 +1504,10 @@ namespace Timbl {
 		 FV_j->ValFreq() >= matrix_clip_freq &&
 		 ( Prestored_metric != metric->type() ||
 		   fabs(metric_matrix->Extract(FV_i,FV_j)) < Epsilon ) ){
-	      double dist = metric->distance( (FeatureValue*)FV_i,
-					      (FeatureValue*)FV_j,
-					      limit );
+	      double dist =
+		metric->distance( reinterpret_cast<FeatureValue*>(FV_i),
+				  reinterpret_cast<FeatureValue*>(FV_j),
+				  limit );
 	      metric_matrix->Assign( FV_i, FV_j, dist );
 	    }
 	  }
@@ -1742,7 +1743,7 @@ namespace Timbl {
     else {
       it->second->IncValFreq( freq );
     }
-    return (TargetValue *)ValuesMap[index];
+    return reinterpret_cast<TargetValue*>(ValuesMap[index]);
   }
 
   TargetValue *Target::MajorityClass() const {
@@ -1756,7 +1757,7 @@ namespace Timbl {
       }
       ++it;
     }
-    return (TargetValue*)result;
+    return reinterpret_cast<TargetValue*>(result);
   }
 
   bool Target::increment_value( TargetValue *TV ){
@@ -1819,14 +1820,4 @@ namespace Timbl {
     return os;
   }
 
-  void Instance::permute( const std::vector<size_t> permutation ){
-    std::vector<FeatureValue *> vals( FV.size(), 0 );
-    for ( size_t i=0; i < FV.size(); ++i ){
-      vals[i] = FV[permutation[i]];
-    }
-    for ( size_t i=0; i < FV.size(); ++i ){
-      FV[i] = vals[i];
-    }
-  }
-
 }
