@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -37,6 +37,8 @@
 #include <cassert>
 
 #include "ticcutils/StringOps.h"
+#include "ticcutils/PrettyPrint.h"
+#include "ticcutils/Timer.h"
 #include "ticcutils/TreeHash.h"
 
 #include "timbl/MsgClass.h"
@@ -400,8 +402,9 @@ namespace Timbl {
       if ( sock_os && sock_os->good() ){
 	return true;
       }
-      else
+      else {
 	FatalError( "connecting streams to socket failed" );
+      }
     }
     return false;
   }
@@ -466,6 +469,104 @@ namespace Timbl {
     }
   }
 
+  string MBLClass::extract_limited_m( size_t lim ){
+    default_order();
+    set_order();
+    string result;
+    MetricType gm = globalMetricOption;
+    result += TiCC::toString( gm );
+    set<size_t> ignore;
+    map<string,set<size_t>> metrics;
+    // using TiCC::operator<<;
+    // cerr << "permutation: " << permutation << endl;
+    for ( size_t k=0; k < num_of_features; ++k ){
+      if ( Features[permutation[k]]->Ignore() ){
+	//	cerr << "Add " << k+1 << " to ignore" << endl;
+	ignore.insert(k+1);
+      }
+      else {
+	MetricType m = Features[permutation[k]]->getMetricType();
+	if ( m != gm ){
+	  metrics[TiCC::toString( m )].insert(k+1);
+	}
+      }
+    }
+    // cerr << "lim=" << lim << " so start ignoring at: "
+    //  	 << lim + ignore.size()<< endl;
+
+    for ( size_t i=lim+ignore.size(); i < num_of_features; ++i ){
+      //      cerr << "Add " << permutation[i]+1 << " to ignore" << endl;
+      ignore.insert( permutation[i]+1 );
+    }
+    if ( !ignore.empty() ){
+      result += ":I";
+      // using TiCC::operator<<;
+      // cerr << "IGNORE bevat:" << ignore << endl;
+      for ( auto it = ignore.begin(); it != ignore.end(); ++it ){
+	size_t value = *it;
+	//	cerr << "START it=" << *it << " value = " << value << endl;
+	size_t steps = 0;
+	for ( ; value <= *ignore.rbegin(); ++value ){
+	  //	  cerr << "value = " << value << endl;
+	  if ( ignore.find(value) == ignore.end() ){
+	    break;
+	  }
+	  ++steps;
+	}
+	// cerr << "END it=" << *it << " value = " << value << endl;
+	// cerr << "STEPS =" << steps << endl;
+	if ( value == *it+1 ){
+	  // so only one value, output it
+	  if ( *it != *ignore.begin() ){
+	    result += ",";
+	  }
+	  result += TiCC::toString(*it) + ",";
+	}
+	else if ( value == *it+2 ){
+	  // so only two values, output them , separated
+	  result += TiCC::toString(*it);
+	  ++it;
+	  result += "," + TiCC::toString(*it);
+	}
+	else {
+	  // a range. output with a hyphen
+	  result += TiCC::toString(*it) + "-" + TiCC::toString( value-1) + ",";
+	  //	  cerr << "advance it met " << steps-1 << endl;
+	  for ( size_t j=0; j < steps-1;++j){
+	    ++it;
+	    if ( it == ignore.end() ){
+	      --it;
+	      break;
+	    }
+	  }
+	}
+      }
+      result.pop_back();
+    }
+    result += ":";
+    for ( const auto& it : metrics ){
+      bool first = true;
+      for ( const auto& ig : it.second ){
+	if ( ignore.find( ig ) == ignore.end() ){
+	  if ( first ){
+	    result += it.first;
+	    first = false;
+	  }
+	  result += TiCC::toString( ig ) + ",";
+	}
+      }
+      if ( result.back() == ',' ){
+	result.pop_back();
+	result.push_back(':');
+      }
+    }
+    while ( result.back() == ':'
+	    || result.back() == ',' ){
+      result.pop_back();
+    }
+    return result;
+  }
+
   void MBLClass::writePermutation( ostream& os ) const {
     os << "Feature Permutation based on "
        << ( Weighting==UserDefined_w?"weightfile":TiCC::toString(TreeOrder, true))
@@ -476,17 +577,6 @@ namespace Timbl {
     os << permutation[num_of_features-1]+1 << " >" << endl;
   }
 
-  inline char *CurTime(){
-    time_t lTime;
-    struct tm *curtime;
-    char *time_string;
-    time(&lTime);
-    curtime = localtime(&lTime);
-    time_string = asctime(curtime);
-    time_string[24] = '\0'; // defeat the newline!
-    return time_string;
-  }
-
   void MBLClass::time_stamp( const char *line, int number ) const {
     if ( !Verbosity(SILENT) ){
       ostringstream ostr;
@@ -498,7 +588,7 @@ namespace Timbl {
       }
       else
 	ostr << "        ";
-      ostr << CurTime();
+      ostr << TiCC::Timer::now();
       Info( ostr.str() );
     }
   }
@@ -686,7 +776,7 @@ namespace Timbl {
       line = TiCC::trim( line );
       if ( line.empty() )
 	continue;
-      if ( line.find( "Feature" ) != 0 ){
+      if ( line.compare( 0, 7, "Feature" ) != 0 ){
 	if ( skip )
 	  continue;
 	else
@@ -816,7 +906,7 @@ namespace Timbl {
 	}
       }
     }
-    while ( result && !is.eof() & !is.bad() );
+    while ( result && !is.eof() && !is.bad() );
     if ( index < num_of_features+1 ){
       Error( "Not enough features matrices in this file " );
       result = false;
@@ -1264,10 +1354,9 @@ namespace Timbl {
   }
 
   bool MBLClass::readWeights( istream& is, WeightType wanted ){
-    set<WeightType> ret_weights;
-    bool result = false;
-    bool old_style = true;
     if ( !ExpInvalid() ){
+      bool old_style = true;
+      bool result = false;
       string Buffer;
       while( getline( is, Buffer ) ) {
 	// A comment starts with '#'
@@ -1345,12 +1434,12 @@ namespace Timbl {
     if ( always || realy_first ){
       // if it's the first time (DBEntropy == 0 ) or
       // if always, we have to (re)calculate everything
-      double Entropy = 0.0, Ratio;
+      double Entropy = 0.0;
       // first get the Database Entropy
       size_t totval = Targets->TotalValues();
       VCarrtype::const_iterator it = Targets->ValuesArray.begin();
       while ( it != Targets->ValuesArray.end() ){
-	Ratio = (*it)->ValFreq() /
+	double Ratio = (*it)->ValFreq() /
 	  (double)totval;
 	if ( Ratio > 0 )
 	  Entropy += Ratio * Log2(Ratio);
@@ -1361,7 +1450,7 @@ namespace Timbl {
     }
     // Loop over the Features, see if the numerics are non-singular
     // and do the statistics for those features where the metric is changed.
-    FeatVal_Stat *feat_status = new FeatVal_Stat[num_of_features];
+    vector<FeatVal_Stat> feat_status(num_of_features);
     bool nothing_changed = true;
     for ( size_t g = 0; g < num_of_features; ++g ) {
       feat_status[g] = Unknown;
@@ -1414,7 +1503,6 @@ namespace Timbl {
       // or Similarity Metrics!
       bool first = true;
       ostringstream ostr1;
-      ostringstream ostr2;
       for ( size_t ff = 0; ff < num_of_features; ++ff ){
 	if ( feat_status[ff] == NotNumeric ){
 	  if ( first ){
@@ -1510,7 +1598,6 @@ namespace Timbl {
 	}
       }
     }
-    delete [] feat_status;
   }
 
   bool MBLClass::writeNamesFile( ostream& os ) const {
@@ -1814,14 +1901,16 @@ namespace Timbl {
 							       ib_offset,
 							       effective_feats );
     tester->init( Inst, effective_feats, ib_offset );
-    size_t CurPos = 0;
     while ( best_distrib ){
-      double dummy = -1.0;
+      double dummy_t = -1.0;
+      size_t dummy_p = 0;
+      // similarity::test() doesn't need CurPos, nor a Threshold
+      // it recalculates the whole vector
       size_t EndPos = tester->test( CurrentFV,
-				    CurPos,
-				    dummy );
+				    dummy_p,
+				    dummy_t );
       if ( EndPos == EffFeat ){
-	// we finished with a certain amount of succes
+	// this should always be true!
 	double Distance = tester->getDistance(EndPos);
 	if ( Distance >= 0.0 ){
 	  string origI;
@@ -1831,7 +1920,6 @@ namespace Timbl {
 				    num_of_features );
 	  }
 	  bestArray.addResult( Distance, best_distrib, origI );
-
 	}
 	else if ( GlobalMetric->type() == DotProduct ){
 	  Error( "The Dot Product metric fails on your data: intermediate result too big to handle," );
@@ -1839,18 +1927,15 @@ namespace Timbl {
 	  FatalError( "timbl terminated" );
 	}
 	else {
-	  Error( "DISTANCE == " + TiCC::toString<double>(Distance) );
+	  Error( "negative similarity DISTANCE: " + TiCC::toString<double>(Distance) );
 	  FatalError( "we are dead" );
 	}
       }
       else {
-	EndPos++; // out of luck, compensate for roll-back
-      }
-      if ( EndPos > 0 ){
-	CurPos = EndPos-1;
-	best_distrib = IB->NextGraphTest( CurrentFV,
-					  CurPos );
+	throw( logic_error( "Similarity testing: test should consider all features" ) );
       }
+      --EndPos;
+      best_distrib = IB->NextGraphTest( CurrentFV, EndPos );
     }
   }
 
