@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -47,8 +47,6 @@ using Common::Log2;
 
 namespace Timbl{
 
-  const double maxSimilarity = std::numeric_limits<int>::max();
-
   double lv_distance( const string& source, const string& target ){
     // code taken from: http://www.merriampark.com/ldcpp.htm
     //    Levenshtein Distance Algorithm: C++ Implementation
@@ -135,7 +133,6 @@ namespace Timbl{
 	string2_unigrams.insert(c);
       }
 
-      set<char>::const_iterator it = string2_unigrams.begin();
       for ( const auto& ug : string2_unigrams ){
 	if ( string1_unigrams.find( ug ) != string1_unigrams.end() )
 	  ++overlap;
