@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -46,6 +46,7 @@ using Common::Log2;
 namespace Timbl{
 
   //#define DBGTEST
+  //#define DBGTEST_DOT
 
   double overlapTestFunction::test( FeatureValue *F,
 				    FeatureValue *G,
@@ -68,7 +69,7 @@ namespace Timbl{
 				      FeatureValue *G,
 				      Feature *Feat ) const {
 #ifdef DBGTEST
-    cerr << toString(Feat->getMetricType()) << "_distance(" << F << "," << G << ") = ";
+    cerr << TiCC::toString(Feat->getMetricType()) << "_distance(" << F << "," << G << ") = ";
 #endif
     double result = Feat->fvDistance( F, G, threshold );
 #ifdef DBGTEST
@@ -95,7 +96,12 @@ namespace Timbl{
 
   TesterClass::TesterClass( const vector<Feature*>& feat,
 			    const vector<size_t>& perm ):
-    _size(feat.size()), features(feat), permutation(perm) {
+    _size(feat.size()),
+    effSize(_size),
+    offSet(0),
+    FV(0),
+    features(feat),
+    permutation(perm) {
     permFeatures.resize(_size,0);
 #ifdef DBGTEST
     cerr << "created TesterClass(" << _size << ")" << endl;
@@ -135,7 +141,7 @@ namespace Timbl{
     for ( size_t i=0; i < _size; ++i ){
       metricTest[i] = 0;
 #ifdef DBGTEST
-      cerr << "set metric[" << i+1 << "]=" << toString(features[i]->getMetricType()) << endl;
+      cerr << "set metric[" << i+1 << "]=" << TiCC::toString(features[i]->getMetricType()) << endl;
 #endif
       if ( features[i]->Ignore() )
 	continue;
@@ -197,67 +203,82 @@ namespace Timbl{
   double innerProduct( FeatureValue *FV,
 		       FeatureValue *G ) {
     double r1, r2, result;
-#ifdef DBGTEST
+#ifdef DBGTEST_DOT
     cerr << "innerproduct " << FV << " x " << G << endl;
 #endif
     if ( FV_to_real( FV, r1 ) &&
 	 FV_to_real( G, r2 ) ){
-#ifdef DBGTEST
+#ifdef DBGTEST_DOT
       cerr << "innerproduct " << r1 << " x " << r2 << endl;
 #endif
       result = r1 * r2;
     }
-    else
+    else {
       result = 0.0;
-#ifdef DBGTEST
+    }
+#ifdef DBGTEST_DOT
     cerr << " resultaat == " << result << endl;
 #endif
     return result;
   }
 
   size_t CosineTester::test( vector<FeatureValue *>& G,
-			     size_t CurPos,
+			     size_t,
 			     double ){
     double denom1 = 0.0;
     double denom2 = 0.0;
     double result = 0.0;
     size_t TrueF;
     size_t i;
-    for ( i=CurPos, TrueF = i + offSet; i < effSize; ++i,++TrueF ){
+    for ( i=0, TrueF = i + offSet; i < effSize; ++i,++TrueF ){
       double W = permFeatures[TrueF]->Weight();
-      denom1 +=  innerProduct( (*FV)[TrueF], (*FV)[TrueF] ) * W;
+      denom1 += innerProduct( (*FV)[TrueF], (*FV)[TrueF] ) * W;
       denom2 += innerProduct( G[i], G[i] ) * W;
       result += innerProduct( (*FV)[TrueF], G[i] ) * W;
     }
     double denom = sqrt( denom1 * denom2 );
     distances[effSize] = result/ (denom + Common::Epsilon);
+#ifdef DBGTEST
+    cerr << "denom1 " << denom1 << endl;
+    cerr << "denom2 " << denom2 << endl;
+    cerr << "denom  " << denom << endl;
+    cerr << "result " << result << endl;
+    cerr << "cosine::test() distance " <<  distances[effSize] << endl;
+#endif
     return effSize;
   }
 
   size_t DotProductTester::test( vector<FeatureValue *>& G,
-				 size_t CurPos,
+				 size_t,
 				 double ) {
-    double result;
     size_t TrueF;
     size_t i;
-    for ( i=CurPos, TrueF = i + offSet; i < effSize; ++i,++TrueF ){
-      result = innerProduct( (*FV)[TrueF], G[i] );
+    for ( i=0, TrueF = i + offSet; i < effSize; ++i,++TrueF ){
+      double result = innerProduct( (*FV)[TrueF], G[i] );
       result *= permFeatures[TrueF]->Weight();
       distances[i+1] = distances[i] + result;
 #ifdef DBGTEST
       cerr << "gewogen result " << result << endl;
-      cerr << "distance[" << i+1 << "]=" <<  distances[i+1] << endl;
+      cerr << "dot::test() distance[" << i+1 << "]=" <<  distances[i+1] << endl;
 #endif
     }
     return effSize;
   }
 
-  double SimilarityTester::getDistance( size_t pos ) const{
+  double CosineTester::getDistance( size_t pos ) const{
 #ifdef DBGTEST
-    cerr << "getDistance, maxSim = " << maxSimilarity << endl;
+    cerr << "getDistance, maxSim = " << 1.0  << endl;
     cerr << " distances[" << pos << "]= " <<  distances[pos] << endl;
 #endif
-    return maxSimilarity - distances[pos];
+    return 1.0 - distances[pos];
+  }
+
+  double DotProductTester::getDistance( size_t pos ) const{
+#ifdef DBGTEST
+    cerr << "getDistance, maxSim = " << std::numeric_limits<int>::max() << endl;
+    cerr << " distances[" << pos << "]= " <<  distances[pos] << endl;
+#endif
+    return (std::numeric_limits<int>::max() - distances[pos])/std::numeric_limits<int>::max();;
   }
 
 }
