@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -39,6 +39,7 @@
 
 #include "config.h"
 #include "ticcutils/CommandLine.h"
+#include "ticcutils/Timer.h"
 #include "timbl/TimblAPI.h"
 
 using namespace std;
@@ -53,6 +54,8 @@ bool Do_LOO = false;
 bool Do_NS = false;
 bool Do_Indirect = false;
 bool Do_Save_Perc = false;
+bool Do_Limit = false;
+size_t limit_val = 0;
 
 string I_Path = "";
 string O_Path = "";
@@ -203,7 +206,7 @@ inline void usage_full(void){
        << "                     then normalize between 0 and 1 (default f=1.0)" << endl
        << "             1:<f> : does the same" << endl
        << "    logProbability : Add 1 to the target Weight, take the 10Log and" << endl
-       << "                     then normalize between 0 and 1 (default f=1.0)" << endl
+       << "                     then normalize between 0 and 1." << endl
        << "             2     : does the same" << endl;
   cerr << "-W f      : calculate and save all Weights in file 'f'" << endl;
   cerr << "+% or -%  : do or don't save test result (%) to file" << endl;
@@ -221,6 +224,8 @@ inline void usage_full(void){
   cerr << "-M n      : size of MaxBests Array" << endl;
   cerr << "-N n      : Number of features (default "
        << TimblAPI::Default_Max_Feats() << ")" << endl;
+  cerr << "--limit l : limit the number of features used to the 'l' with the highest weights." << endl;
+  cerr << "            (will restart Timbl with an adapted -m option)" << endl;
   cerr << "--Treeorder=<value>      : ordering of the Tree :" << endl;
   cerr << "       DO: none" << endl;
   cerr << "       GRO: using GainRatio" << endl;
@@ -321,6 +326,17 @@ void Preset_Values( TiCC::CL_Options& opts ){
 	throw( hardExit() ); // no chance to proceed
       }
   }
+  if ( opts.extract( "limit", value ) ){
+    Do_Limit = true;
+    if ( !TiCC::stringTo<size_t>( value, limit_val ) ){
+      cerr << "illegal --limit value: " << value << endl;
+      throw( hardExit() ); // no chance to proceed
+    }
+    if ( Do_CV ){
+      cerr << "--limit is not implemented for --cross-validation" << endl;
+      throw( hardExit() ); // no chance to proceed
+    }
+  }
   if ( opts.extract( 'P', value ) ){
     I_Path = value;
   }
@@ -349,7 +365,6 @@ void Preset_Values( TiCC::CL_Options& opts ){
 }
 
 void Adjust_Default_Values( TiCC::CL_Options& opts ){
-  string value;
   if ( !opts.is_present( 'm' ) ){
     opts.insert( 'm', "O", false );
     // Default Metric = Overlap
@@ -619,7 +634,7 @@ void Do_Test( TimblAPI *Run ){
 	cerr << "Warning: Skipped a line from indirect testfile:\n'"
 	     << tmp_line << "'" << endl;
 	if ( TestFile == "" )
-	  cerr << "missing a Testfile name " << endl;
+	  cerr << "missing a Testfile name (-t option)" << endl;
       }
       else if ( Run->SetIndirectOptions( opts ) ){
 	Default_Output_Names( opts );
@@ -670,19 +685,15 @@ bool checkOutputFile( const string& name ){
 
 int main(int argc, char *argv[]){
   try {
-    struct tm *curtime;
-    time_t Time;
     // Start.
     //
     cerr << "TiMBL " << TimblAPI::VersionInfo()
-	 << " (c) CLST/ILK/CLIPS 1998 - 2016.\n"
+	 << " (c) CLST/ILK/CLIPS 1998 - 2018.\n"
 	 << "Tilburg Memory Based Learner\n"
 	 << "Centre for Language and Speech Technology, Radboud University\n"
 	 << "Induction of Linguistic Knowledge Research Group, Tilburg University\n"
 	 << "CLiPS Computational Linguistics Group, University of Antwerp" << endl;
-    time(&Time);
-    curtime = localtime(&Time);
-    cerr << asctime(curtime) << endl;
+    cerr << TiCC::Timer::now() << endl << endl;
     if ( argc <= 1 ){
       usage();
       return 1;
@@ -746,6 +757,21 @@ int main(int argc, char *argv[]){
 	if ( WgtOutFile != "" )
 	  Run->SetOptions( "ALL_WEIGHTS: true" );
 	if ( Run->Prepare( dataFile ) ){
+	  if ( Do_Limit ){
+	    if ( Run->NumOfFeatures() < limit_val ){
+	      cerr << "value of --limit is larger then the number of features!"
+		   << endl;
+	      return 32;
+	    }
+	    string m_val = Run->extract_limited_m( limit_val );
+	    //	    cerr << endl << endl << "NEW M: " << m_val << endl << endl;
+	    opts.extract( 'm' );
+	    opts.insert( 'm', m_val, true );
+	    cerr << "\t--limit=" << limit_val << " is specified, so we retrain "
+		 << "the data with option: -m" << m_val << endl;
+	    delete Run;
+	    Run = new TimblAPI( opts );
+	  }
 	  if ( WgtOutFile != "" ) {
 	    Run->SaveWeights( WgtOutFile );
 	  }
@@ -823,24 +849,25 @@ int main(int argc, char *argv[]){
 	  Run->WriteMatrices( MatrixOutFile );
 	}
       }
-      delete Run;
       if ( !do_test || !Run->isValid() ){
+	delete Run;
 	return EXIT_FAILURE;
       }
+      delete Run;
     }
     return EXIT_SUCCESS;
   }
   catch( softExit& e ){
     return EXIT_SUCCESS;
   }
-  catch(std::string& what){
+  catch( const std::string& what ){
     cerr << what << ", sorry" << endl;
   }
-  catch(std::bad_alloc){
+  catch( const std::bad_alloc&){
     cerr << "ran out of memory somewhere" << endl;
     cerr << "timbl terminated, Sorry for that" << endl;
   }
-  catch(std::exception& e){
+  catch( const std::exception& e ){
     cerr << e.what() << ", sorry" << endl;
   }
   return EXIT_FAILURE;
