@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -646,28 +646,19 @@ namespace Timbl {
     }
     else
       return false;
-
   }
 
   bool TimblAPI::ShowStatistics( ostream& os ) const{
     return Valid() && pimpl->showStatistics( os );
   }
 
-  bool TimblAPI::StartServer( const int , const int ){
-    return false;
-  }
-
-  bool TimblAPI::StartMultiServer( const string& ){
-    return false;
-  }
-
-  bool TimblAPI::SetThreads( int c ){
+  string TimblAPI::extract_limited_m( int lim ) const {
     if ( Valid() ){
-      pimpl->Clones( c );
-      return true;
+      return pimpl->extract_limited_m( lim );
+    }
+    else {
+      return "error";
     }
-    else
-      return false;
   }
 
   string TimblAPI::VersionInfo( bool full ){
@@ -675,8 +666,16 @@ namespace Timbl {
     return Common::VersionInfo( full );
   }
 
-  int TimblAPI::Default_Max_Feats() {
+  size_t TimblAPI::Default_Max_Feats(){
     return Common::DEFAULT_MAX_FEATS;
   }
 
+  size_t TimblAPI::NumOfFeatures() const {
+    if ( Valid() ){
+      return pimpl->NumOfFeatures();
+    }
+    else
+      return -1;
+  }
+
 }
