@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -75,7 +75,7 @@ using namespace std;
 namespace Timbl {
 
   const string timbl_short_opts = "a:b:B:c:C:d:De:f:F:G::hHi:I:k:l:L:m:M:n:N:o:O:p:P:q:QR:s::t:T:u:U:v:Vw:W:xX:Z%";
-  const string timbl_long_opts = ",Beam:,clones:,Diversify,occurrences:,sloppy::,silly::,Threshold:,Treeorder:,matrixin:,matrixout:,version,help";
+  const string timbl_long_opts = ",Beam:,clones:,Diversify,occurrences:,sloppy::,silly::,Threshold:,Treeorder:,matrixin:,matrixout:,version,help,limit:";
   const string timbl_serv_short_opts = "C:d:G::k:l:L:p:Qv:x";
   const string timbl_indirect_opts = "d:e:G:k:L:m:o:p:QR:t:v:w:x%";
 
@@ -243,6 +243,8 @@ namespace Timbl {
     default:
       FatalError( "You may not split experiments for Special cases like " +
 		  TiCC::toString(algorithm) );
+      exit(EXIT_FAILURE); // never reached, (FatalError throws) but satisfies
+      // scan-build
     }
     *result = *this;
     if ( OptParams ){
@@ -400,7 +402,6 @@ namespace Timbl {
 	    else {
 	      TiCC::Timer prepT;
 	      prepT.start();
-	      bool found;
 	      bool go_on = true;
 	      if ( !Verbosity(SILENT) ){
 		Info( "Phase 1: Reading Datafile: " + FileName );
@@ -414,7 +415,7 @@ namespace Timbl {
 		  if (( stats.dataLines() % Progress() ) == 0)
 		    time_stamp( "Examining: ", stats.dataLines() );
 		}
-		found = false;
+		bool found = false;
 		while ( !found &&
 			nextLine( datafile, Buffer ) ){
 		  found = chopLine( Buffer );
@@ -585,7 +586,6 @@ namespace Timbl {
 	    Info( "\nPhase 3: Learning from Datafile: " + CurrentDataFile );
 	    time_stamp( "Start:     ", 0 );
 	  }
-	  string Buffer;
 	  // Open the file.
 	  //
 	  ifstream datafile( CurrentDataFile, ios::in);
@@ -858,15 +858,10 @@ namespace Timbl {
 
   void TimblExperiment::show_progress( ostream& os,
 				       time_t start, unsigned int line ){
-    char time_string[26];
-    struct tm *curtime;
-    time_t Time;
-    time_t SecsUsed;
-    time_t EstimatedTime;
-    double Estimated;
     int local_progress = Progress();
     if ( ( (line % local_progress ) == 0) || ( line <= 10 ) ||
 	 ( line == 100 || line == 1000 || line == 10000 ) ){
+      time_t Time;
       time(&Time);
       if ( line == 1000 ){
 	// check if we are slow, if so, change progress value
@@ -879,28 +874,23 @@ namespace Timbl {
 	  // quit slow !
 	  Progress( 10000 );
       }
-      curtime = localtime(&Time);
       if ( exp_name != "" )
 	os  << "-" << exp_name << "-";
       os << "Tested: ";
       os.width(6);
       os.setf(ios::right, ios::adjustfield);
-      strcpy( time_string, asctime(curtime));
-      time_string[24] = '\0';
-      os << line << " @ " << time_string;
-
-      // Estime time until Estimate.
+      os << line << " @ " << TiCC::Timer::now();
+      // Estimate time until Estimate.
       //
-      if ( Estimate() > 0 &&  (unsigned int)Estimate() < line ) {
-	SecsUsed = Time - start;
+      if ( Estimate() > 0 &&  (unsigned int)Estimate() > line ) {
+	time_t SecsUsed = Time - start;
 	if ( SecsUsed > 0 ) {
-	  Estimated = (SecsUsed / (float)line) *
+	  double Estimated = (SecsUsed / (float)line) *
 	    (float)Estimate();
-	  EstimatedTime = (long)Estimated + start;
-	  os << ", ";
-	  strcpy(time_string, ctime(&EstimatedTime));
-	  time_string[24] = '\0';
-	  os << Estimate() << ": " << time_string;
+	  time_t EstimatedTime = (long)Estimated + start;
+	  string time_string = ctime(&EstimatedTime);
+	  time_string.pop_back(); // the newline from ctime
+	  os << ", " << Estimate() << ": " << time_string;
 	}
       }
       os << endl;
@@ -910,17 +900,12 @@ namespace Timbl {
   bool IB2_Experiment::show_learn_progress( ostream& os,
 					    time_t start,
 					    size_t added ){
-    char time_string[26];
-    struct tm *curtime;
-    time_t Time;
-    time_t SecsUsed;
-    time_t EstimatedTime;
-    double Estimated;
     int local_progress = Progress();
     unsigned int lines = stats.dataLines();
     unsigned int line = lines - IB2_offset() ;
     if ( ( (line % local_progress ) == 0) || ( line <= 10 ) ||
 	 ( line == 100 || line == 1000 || line == 10000 ) ){
+      time_t Time;
       time(&Time);
       if ( line == 100 ){
 	// check if we are slow, if so, change progress value
@@ -939,28 +924,24 @@ namespace Timbl {
 	  // quit slow !
 	  Progress( 10000 );
       }
-      curtime = localtime(&Time);
       if ( exp_name != "" )
 	os  << "-" << exp_name << "-";
       os << "Learning:  ";
       os.width(6);
       os.setf(ios::right, ios::adjustfield);
-      strcpy( time_string, asctime(curtime));
-      time_string[24] = '\0';
-      os << lines << " @ " << time_string;
+      os << lines << " @ " << TiCC::Timer::now();
       os << "\t added:" << added;
       // Estime time until Estimate.
       //
-      if ( Estimate() > 0 && (unsigned int)Estimate() < lines ) {
-	SecsUsed = Time - start;
+      if ( Estimate() > 0 && (unsigned int)Estimate() > lines ) {
+	time_t SecsUsed = Time - start;
 	if ( SecsUsed > 0 ) {
-	  Estimated = (SecsUsed / (float)line) *
+	  double Estimated = (SecsUsed / (float)line) *
 	    ( (float)Estimate() - IB2_offset() );
-	  EstimatedTime = (long)Estimated + start;
-	  os << "\t, ";
-	  strcpy(time_string, ctime(&EstimatedTime));
-	  time_string[24] = '\0';
-	  os << Estimate() << ": " << time_string;
+	  time_t EstimatedTime = (long)Estimated + start;
+	  string time_string = ctime(&EstimatedTime);
+	  time_string.pop_back(); // the newline from ctime
+	  os << "\t, " << Estimate() << ": " << time_string;
 	}
       }
       os << endl;
@@ -1072,9 +1053,9 @@ namespace Timbl {
       int OldPrec = outfile.precision(DBL_DIG-1);
       outfile.setf(ios::showpoint);
       outfile.width(8);
-      if ( GlobalMetric->isSimilarityMetric() )
-	outfile << " " << maxSimilarity-Distance;
-      else
+      // if ( GlobalMetric->isSimilarityMetric() )
+      // 	outfile << " " << GlobalMetric->get_max_similarity() - Distance;
+      // else
 	outfile << " " << Distance;
       outfile.precision(OldPrec);
     }
@@ -1233,8 +1214,6 @@ namespace Timbl {
 
   bool IB2_Experiment::Expand_N( const string& FileName ){
     bool result = true;
-    size_t Added = 0;
-    size_t TotalAdded = 0;
     if ( ExpInvalid() ){
       result = false;
     }
@@ -1289,6 +1268,8 @@ namespace Timbl {
 	    time_stamp( "Start:     ", stats.dataLines() );
 	  }
 	  bool found;
+	  size_t TotalAdded = 0;
+	  size_t Added = 0;
 	  initExperiment();
 	  do {
 	    // The next Instance to store.
@@ -1431,7 +1412,6 @@ namespace Timbl {
   }
 
   bool TimblExperiment::checkLine( const string& line ){
-    size_t i;
     bool result = false;
     if ( !ExpInvalid() &&
 	 ConfirmOptions() ) {
@@ -1439,7 +1419,8 @@ namespace Timbl {
       InputFormatType IF = InputFormat();
       if ( IF == UnknownInputFormat )
 	IF = getInputFormat( line );
-      if ( (i = countFeatures( line, IF )) != NumOfFeatures() ){
+      size_t i = countFeatures( line, IF );
+      if ( i != NumOfFeatures() ){
 	if ( i > 0 )
 	  Warning( "mismatch between number of features in testline " +
 		   line + " and the Instancebase (" + TiCC::toString<size_t>(i)
@@ -1583,8 +1564,9 @@ namespace Timbl {
 	delete ResultDist;
 	ResultDist = ResultDist2;
       }
-      else
+      else {
 	delete ResultDist2;
+      }
     }
 
     exact = fabs(Distance) < Epsilon ;
@@ -1816,11 +1798,10 @@ namespace Timbl {
 
   bool threadBlock::readLines( istream& is  ){
     bool result = true;
-    bool goon = true;
     for ( size_t i=0; i < size; ++i ){
       exps[i].Buffer = "";
       int cnt;
-      goon = exps[0].exp->nextLine( is, exps[i].Buffer, cnt );
+      bool goon = exps[0].exp->nextLine( is, exps[i].Buffer, cnt );
       exps[i].lineNo += cnt;
       if ( !goon && i == 0 )
 	result = false;
@@ -2065,11 +2046,6 @@ namespace Timbl {
     return OptParams->definitive_options( this );
   }
 
-  bool TimblExperiment::DefaultOptions(){
-    OptParams->set_default_options();
-    return true;
-  }
-
   bool TimblExperiment::ShowOptions( ostream& os ){
     return ( ConfirmOptions() &&
 	     MBLClass::ShowOptions( os ) );
@@ -2422,9 +2398,8 @@ namespace Timbl {
 	Info( "Phase 2: Building index on Datafile: " + file_name );
 	time_stamp( "Start:     ", 0 );
       }
-      bool found;
       bool go_on = true;
-      while( go_on ){
+      while ( go_on ){
 	// The next Instance to store.
 	//	cerr << "line at pos " << cur_pos << " : " << Buffer << endl;
 	chopped_to_instance( TrainWords );
@@ -2441,7 +2416,7 @@ namespace Timbl {
 	}
 	if ((stats.dataLines() % Progress() ) == 0)
 	  time_stamp( "Indexing:  ", stats.dataLines() );
-	found = false;
+	bool found = false;
 	while ( !found &&
 		( cur_pos = datafile.tellg(),
 		  nextLine( datafile, Buffer ) ) ){
@@ -2484,7 +2459,6 @@ namespace Timbl {
 	Info( "Phase 2: Building multi index on Datafile: " + file_name );
 	time_stamp( "Start:     ", 0 );
       }
-      bool found;
       bool go_on = true;
       while( go_on ){
 	// The next Instance to store.
@@ -2501,9 +2475,10 @@ namespace Timbl {
 	  mi[fv1].insert( cur_pos );
 	  fmIndex[fv0] = mi;
 	}
-	if ((stats.dataLines() % Progress() ) == 0)
+	if ( (stats.dataLines() % Progress() ) == 0 ){
 	  time_stamp( "Indexing:  ", stats.dataLines() );
-	found = false;
+	}
+	bool found = false;
 	while ( !found &&
 		( cur_pos = datafile.tellg(),
 		  nextLine( datafile, Buffer ) ) ){
