@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   ILK   - Tilburg University
   CLST  - Radboud University
   CLiPS - University of Antwerp
@@ -109,7 +109,7 @@ namespace Timbl {
     { "BINARY", "Sparse Binary" },
     { "SPARSE", "Sparse" } };
 
-  const string VerbosityName[][2] = { { "Unknown", "erronous" },
+  const string VerbosityName[][2] = { { "Unknown", "erroneous" },
 				      { "S", "Silent" },
 				      { "O", "Options" },
 				      { "F", "Feature_Statistics" },
